@@ -23,6 +23,10 @@ When use `input_group`, you needs to provide the ``name`` parameter in each inpu
 
 By default, the user can submit empty input value. If the user must provide a non-empty input value, you need to pass ``required=True`` to the input function (some input functions do not support the ``required`` parameter)
 
+The input functions in this module is blocking, and the input form will be destroyed after successful submission.
+If you want the form to always be displayed on the page and receive input continuously,
+you can consider the :doc:`pin <./pin>` module.
+
 Functions list
 -----------------
 
@@ -46,6 +50,9 @@ Functions list
    * - `radio <pywebio.input.radio>`
      - Radio
 
+   * - `slider <pywebio.input.slider>`
+     - Slider
+
    * - `actions <pywebio.input.actions>`
      - Actions selection
 
@@ -55,18 +62,21 @@ Functions list
    * - `input_group <pywebio.input.input_group>`
      - Input group
 
+   * - `input_update <pywebio.input.input_update>`
+     - Update input item
+
 
 Functions doc
 --------------
 """
-
+import os.path
 import logging
-from base64 import b64decode
 from collections.abc import Mapping
 
-from .io_ctrl import single_input, input_control, output_register_callback
+from .io_ctrl import single_input, input_control, output_register_callback, send_msg, single_input_kwargs
 from .session import get_current_session, get_current_task_id
 from .utils import Setter, is_html_safe_value, parse_file_size
+from .platform import utils as platform_setting
 
 logger = logging.getLogger(__name__)
 
@@ -84,7 +94,7 @@ SELECT = 'select'
 TEXTAREA = 'textarea'
 
 __all__ = ['TEXT', 'NUMBER', 'FLOAT', 'PASSWORD', 'URL', 'DATE', 'TIME', 'input', 'textarea', 'select',
-           'checkbox', 'radio', 'actions', 'file_upload', 'input_group']
+           'checkbox', 'radio', 'actions', 'file_upload', 'slider', 'input_group', 'input_update']
 
 
 def _parse_args(kwargs, excludes=()):
@@ -97,14 +107,25 @@ def _parse_args(kwargs, excludes=()):
     """
     kwargs = {k: v for k, v in kwargs.items() if v is not None and k not in excludes}
     assert is_html_safe_value(kwargs.get('name', '')), '`name` can only contains a-z、A-Z、0-9、_、-'
+
     kwargs.update(kwargs.get('other_html_attrs', {}))
     kwargs.pop('other_html_attrs', None)
+
+    if kwargs.get('validate'):
+        kwargs['onblur'] = True
     valid_func = kwargs.pop('validate', lambda _: None)
-    return kwargs, valid_func
+
+    if kwargs.get('onchange'):
+        onchange_func = kwargs['onchange']
+        kwargs['onchange'] = True
+    else:
+        onchange_func = lambda _: None
+
+    return kwargs, valid_func, onchange_func
 
 
-def input(label='', type=TEXT, *, validate=None, name=None, value=None, action=None, placeholder=None, required=None,
-          readonly=None, datalist=None, help_text=None, **other_html_attrs):
+def input(label='', type=TEXT, *, validate=None, name=None, value=None, action=None, onchange=None, placeholder=None,
+          required=None, readonly=None, datalist=None, help_text=None, **other_html_attrs):
     r"""Text input
 
     :param str label: Label of input field.
@@ -170,6 +191,11 @@ def input(label='', type=TEXT, *, validate=None, name=None, value=None, action=N
 
         Note: When using :ref:`Coroutine-based session <coroutine_based_session>` implementation, the ``callback`` function can be a coroutine function.
 
+    :param callable onchange: A callback function which will be called when the value of this input field changed.
+
+       The ``onchange`` callback is invoked with one argument, the current value of input field.
+       A typical usage scenario of ``onchange`` is to update other input item by using `input_update()`
+
     :param str placeholder: A hint to the user of what can be entered in the input. It will appear in the input field when it has no value set.
     :param bool required: Whether a value is required for the input to be submittable, default is ``False``
     :param bool readonly: Whether the value is readonly(not editable)
@@ -179,15 +205,12 @@ def input(label='', type=TEXT, *, validate=None, name=None, value=None, action=N
     :return: The value that user input.
     """
 
-    item_spec, valid_func = _parse_args(locals(), excludes=('action',))
+    item_spec, valid_func, onchange_func = _parse_args(locals(), excludes=('action',))
 
     # check input type
     allowed_type = {TEXT, NUMBER, FLOAT, PASSWORD, URL, DATE, TIME}
     assert type in allowed_type, 'Input type not allowed.'
 
-    if type == FLOAT:
-        item_spec['type'] = TEXT
-
     value_setter = None
     if action:
         label, callback = action
@@ -215,18 +238,13 @@ def input(label='', type=TEXT, *, validate=None, name=None, value=None, action=N
         if value_setter is not None and value_setter.label == d:
             return value_setter.value
 
-        if type == NUMBER:
-            d = int(d)
-        elif type == FLOAT:
-            d = float(d)
-
         return d
 
-    return single_input(item_spec, valid_func, preprocess_func)
+    return single_input(item_spec, valid_func, preprocess_func, onchange_func)
 
 
 def textarea(label='', *, rows=6, code=None, maxlength=None, minlength=None, validate=None, name=None, value=None,
-             placeholder=None, required=None, readonly=None, help_text=None, **other_html_attrs):
+             onchange=None, placeholder=None, required=None, readonly=None, help_text=None, **other_html_attrs):
     r"""Text input area (multi-line text input)
 
     :param int rows: The number of visible text lines for the input area. Scroll bar will be used when content exceeds.
@@ -248,13 +266,13 @@ def textarea(label='', *, rows=6, code=None, maxlength=None, minlength=None, val
 
         Some commonly used Codemirror options are listed :ref:`here <codemirror_options>`.
 
-    :param - label, validate, name, value, placeholder, required, readonly, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
+    :param - label, validate, name, value, onchange, placeholder, required, readonly, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
     :return: The string value that user input.
     """
-    item_spec, valid_func = _parse_args(locals())
+    item_spec, valid_func, onchange_func = _parse_args(locals())
     item_spec['type'] = TEXTAREA
 
-    return single_input(item_spec, valid_func, lambda d: d)
+    return single_input(item_spec, valid_func, lambda d: d, onchange_func)
 
 
 def _parse_select_options(options):
@@ -288,7 +306,7 @@ def _set_options_selected(options, value):
     return options
 
 
-def select(label='', options=None, *, multiple=None, validate=None, name=None, value=None, required=None,
+def select(label='', options=None, *, multiple=None, validate=None, name=None, value=None, onchange=None, required=None,
            help_text=None, **other_html_attrs):
     r"""Drop-down selection
 
@@ -318,44 +336,43 @@ def select(label='', options=None, *, multiple=None, validate=None, name=None, v
        You can also set the initial selected option by setting the ``selected`` field in the ``options`` list item.
     :type value: list or str
     :param bool required: Whether to select at least one item, only available when ``multiple=True``
-    :param - label, validate, name, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
+    :param - label, validate, name, onchange, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
     :return: If ``multiple=True``, return a list of the values in the ``options`` selected by the user; otherwise, return the single value selected by the user.
     """
     assert options is not None, 'Required `options` parameter in select()'
 
-    item_spec, valid_func = _parse_args(locals(), excludes=['value'])
+    item_spec, valid_func, onchange_func = _parse_args(locals(), excludes=['value'])
     item_spec['options'] = _parse_select_options(options)
     if value is not None:
         item_spec['options'] = _set_options_selected(item_spec['options'], value)
     item_spec['type'] = SELECT
 
-    return single_input(item_spec, valid_func, lambda d: d)
+    return single_input(item_spec, valid_func=valid_func, preprocess_func=lambda d: d, onchange_func=onchange_func)
 
 
-def checkbox(label='', options=None, *, inline=None, validate=None, name=None, value=None, help_text=None,
-             **other_html_attrs):
+def checkbox(label='', options=None, *, inline=None, validate=None, name=None, value=None, onchange=None,
+             help_text=None, **other_html_attrs):
     r"""A group of check box that allowing single values to be selected/deselected.
 
     :param list options: List of options. The format is the same as the ``options`` parameter of the `select()` function
     :param bool inline: Whether to display the options on one line. Default is ``False``
     :param list value: The value list of the initial selected items.
        You can also set the initial selected option by setting the ``selected`` field in the ``options`` list item.
-    :param - label, validate, name, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
+    :param - label, validate, name, onchange, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
     :return: A list of the values in the ``options`` selected by the user
     """
     assert options is not None, 'Required `options` parameter in checkbox()'
 
-    item_spec, valid_func = _parse_args(locals())
+    item_spec, valid_func, onchange_func = _parse_args(locals(), excludes=['value'])
     item_spec['options'] = _parse_select_options(options)
     if value is not None:
-        del item_spec['value']
         item_spec['options'] = _set_options_selected(item_spec['options'], value)
     item_spec['type'] = CHECKBOX
 
-    return single_input(item_spec, valid_func, lambda d: d)
+    return single_input(item_spec, valid_func, lambda d: d, onchange_func)
 
 
-def radio(label='', options=None, *, inline=None, validate=None, name=None, value=None, required=None,
+def radio(label='', options=None, *, inline=None, validate=None, name=None, value=None, onchange=None, required=None,
           help_text=None, **other_html_attrs):
     r"""A group of radio button. Only a single button can be selected.
 
@@ -364,12 +381,12 @@ def radio(label='', options=None, *, inline=None, validate=None, name=None, valu
     :param str value: The value of the initial selected items.
        You can also set the initial selected option by setting the ``selected`` field in the ``options`` list item.
     :param bool required: whether to must select one option. (the user can select nothing option by default)
-    :param - label, validate, name, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
+    :param - label, validate, name, onchange, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
     :return: The value of the option selected by the user, if the user does not select any value, return ``None``
     """
     assert options is not None, 'Required `options` parameter in radio()'
 
-    item_spec, valid_func = _parse_args(locals())
+    item_spec, valid_func, onchange_func = _parse_args(locals())
     item_spec['options'] = _parse_select_options(options)
     if value is not None:
         del item_spec['value']
@@ -379,7 +396,7 @@ def radio(label='', options=None, *, inline=None, validate=None, name=None, valu
         item_spec['options'][-1]['required'] = required
     item_spec['type'] = RADIO
 
-    return single_input(item_spec, valid_func, lambda d: d)
+    return single_input(item_spec, valid_func, lambda d: d, onchange_func)
 
 
 def _parse_action_buttons(buttons):
@@ -494,11 +511,11 @@ def actions(label='', buttons=None, name=None, help_text=None):
     """
     assert buttons is not None, 'Required `buttons` parameter in actions()'
 
-    item_spec, valid_func = _parse_args(locals())
+    item_spec, valid_func, onchange_func = _parse_args(locals())
     item_spec['type'] = 'actions'
     item_spec['buttons'] = _parse_action_buttons(buttons)
 
-    return single_input(item_spec, valid_func, lambda d: d)
+    return single_input(item_spec, valid_func, lambda d: d, onchange_func)
 
 
 def file_upload(label='', accept=None, name=None, placeholder='Choose file', multiple=False, max_size=0,
@@ -538,27 +555,54 @@ def file_upload(label='', accept=None, name=None, placeholder='Choose file', mul
 
     .. note::
     
-        If uploading large files, please pay attention to the file upload size limit setting of the web framework. When using :func:`start_server <pywebio.platform.start_server>` to start the PyWebIO application, the maximum file size to be uploaded allowed by the web framework can be set through the `websocket_max_message_size` parameter
+        If uploading large files, please pay attention to the file upload size limit setting of the web framework.
+        When using :func:`start_server() <pywebio.platform.tornado.start_server>`/:func:`path_deploy() <pywebio.platform.path_deploy>` to start the PyWebIO application,
+        the maximum file size to be uploaded allowed by the web framework can be set through the ``max_payload_size`` parameter.
 
     """
-    item_spec, valid_func = _parse_args(locals())
+    item_spec, valid_func, onchange_func = _parse_args(locals())
     item_spec['type'] = 'file'
-    item_spec['max_size'] = parse_file_size(max_size)
-    item_spec['max_total_size'] = parse_file_size(max_total_size)
+    item_spec['max_size'] = parse_file_size(max_size) or platform_setting.MAX_PAYLOAD_SIZE
+    item_spec['max_total_size'] = parse_file_size(max_total_size) or platform_setting.MAX_PAYLOAD_SIZE
 
-    def read_file(data):  # data: None or [{'filename':, 'dataurl', 'mime_type', 'last_modified'}, ...]
-        for d in data:
-            try:
-                _, encoded = d['dataurl'].split(",", 1)
-            except ValueError:
-                encoded = ''
-            d['content'] = b64decode(encoded)
+    if platform_setting.MAX_PAYLOAD_SIZE:
+        if item_spec['max_size'] > platform_setting.MAX_PAYLOAD_SIZE or \
+                item_spec['max_total_size'] > platform_setting.MAX_PAYLOAD_SIZE:
+            raise ValueError('The `max_size` and `max_total_size` value can not exceed the backend payload size limit. '
+                             'Please increase the `max_total_size` of `start_server()`/`path_deploy()`')
+
+    def read_file(data):
+        for file in data:
+            # Security fix: to avoid interpreting file name as path
+            file['filename'] = os.path.basename(file['filename'])
 
         if not multiple:
             return data[0] if len(data) >= 1 else None
         return data
 
-    return single_input(item_spec, valid_func, read_file)
+    return single_input(item_spec, valid_func, read_file, onchange_func)
+
+
+def slider(label='', *, name=None, value=0, min_value=0, max_value=100, step=1, validate=None, onchange=None,
+           required=None, help_text=None, **other_html_attrs):
+    r"""Range input.
+
+    :param int/float value: The initial value of the slider.
+    :param int/float min_value: The minimum permitted value.
+    :param int/float max_value: The maximum permitted value.
+    :param int step: The stepping interval.
+       Only available when ``value``, ``min_value`` and ``max_value`` are all integer.
+    :param - label, name, validate, onchange, required, help_text, other_html_attrs: Those arguments have the same meaning as for `input()`
+    :return int/float: If one of ``value``, ``min_value`` and ``max_value`` is float,
+       the return value is a float, otherwise an int is returned.
+    """
+    item_spec, valid_func, onchange_func = _parse_args(locals())
+    item_spec['type'] = 'slider'
+    item_spec['float'] = any(isinstance(i, float) for i in (value, min_value, max_value))
+    if item_spec['float']:
+        item_spec['step'] = 'any'
+
+    return single_input(item_spec, valid_func, lambda d: d, onchange_func)
 
 
 def input_group(label='', inputs=None, validate=None, cancelable=False):
@@ -600,28 +644,22 @@ def input_group(label='', inputs=None, validate=None, cancelable=False):
     spec_inputs = []
     preprocess_funcs = {}
     item_valid_funcs = {}
+    onchange_funcs = {}
     for single_input_return in inputs:
-        try:
-            # 协程模式下，单项输入为协程对象，可以通过send(None)来获取传入单项输入的参数字典
-            # In the coroutine mode, the item of `inputs` is coroutine object. using `send(None)` to get the single input function's parameter dict.
-            single_input_return.send(None)
-        except StopIteration as e:
-            input_kwargs = e.args[0]
-        except AttributeError:
-            input_kwargs = single_input_return
-        else:
-            raise RuntimeError("Can't get kwargs from single input")
+        input_kwargs = single_input_kwargs(single_input_return)
 
         assert all(
             k in (input_kwargs or {})
-            for k in ('item_spec', 'preprocess_func', 'valid_func')
+            for k in ('item_spec', 'preprocess_func', 'valid_func', 'onchange_func')
         ), "`inputs` value error in `input_group`. Did you forget to add `name` parameter in input function?"
 
         input_name = input_kwargs['item_spec']['name']
+        assert input_name, "`name` can not be empty!"
         if input_name in preprocess_funcs:
-            raise ValueError("Can't use same `name`:%s in different input in input group!!" % input_name)
+            raise ValueError('Duplicated input item name "%s" in same input group!' % input_name)
         preprocess_funcs[input_name] = input_kwargs['preprocess_func']
         item_valid_funcs[input_name] = input_kwargs['valid_func']
+        onchange_funcs[input_name] = input_kwargs['onchange_func']
         spec_inputs.append(input_kwargs['item_spec'])
 
     if all('auto_focus' not in i for i in spec_inputs):  # No `auto_focus` parameter is set for each input item
@@ -632,5 +670,60 @@ def input_group(label='', inputs=None, validate=None, cancelable=False):
                 break
 
     spec = dict(label=label, inputs=spec_inputs, cancelable=cancelable)
-    return input_control(spec, preprocess_funcs=preprocess_funcs, item_valid_funcs=item_valid_funcs,
+    return input_control(spec, preprocess_funcs=preprocess_funcs,
+                         item_valid_funcs=item_valid_funcs,
+                         onchange_funcs=onchange_funcs,
                          form_valid_funcs=validate)
+
+
+def parse_input_update_spec(spec):
+    for key in spec:
+        assert key not in {'action', 'buttons', 'code', 'inline', 'max_size', 'max_total_size', 'multiple', 'name',
+                           'onchange', 'type', 'validate'}, '%r can not be updated' % key
+
+    attributes = dict((k, v) for k, v in spec.items() if v is not None)
+    if 'options' in spec:
+        attributes['options'] = _parse_select_options(spec['options'])
+    return attributes
+
+
+def input_update(name=None, **spec):
+    """Update attributes of input field.
+    This function can only be called in ``onchange`` callback of input functions.
+
+    :param str name: The ``name`` of the target input item.
+       Optional, default is the name of input field which triggers ``onchange``
+    :param spec: The input parameters need to be updated.
+       Note that those parameters can not be updated:
+       ``type``, ``name``, ``validate``, ``action``, ``code``, ``onchange``, ``multiple``
+
+    An example of implementing dependent input items in an input group:
+
+    .. exportable-codeblock::
+        :name: input-update
+        :summary: Dependent input items in input group
+
+        country2city = {
+            'China': ['Beijing', 'Shanghai', 'Hong Kong'],
+            'USA': ['New York', 'Los Angeles', 'San Francisco'],
+        }
+        countries = list(country2city.keys())
+        location = input_group("Select a location", [
+            select('Country', options=countries, name='country',
+                   onchange=lambda c: input_update('city', options=country2city[c])),
+            select('City', options=country2city[countries[0]], name='city'),
+        ])
+        put_text(location)  # ..demo-only
+    """
+    task_id = get_current_task_id()
+    k = 'onchange_trigger-' + task_id
+    if k not in get_current_session().internal_save:
+        raise RuntimeError("`input_update()` can only be called in `onchange` callback.")
+    trigger_name = get_current_session().internal_save[k]
+
+    if name is None:
+        name = trigger_name
+
+    attributes = parse_input_update_spec(spec)
+
+    send_msg('update_input', dict(target_name=name, attributes=attributes))
