@@ -8,8 +8,9 @@ from urllib.parse import urlparse
 
 from aiohttp import web
 
+from .remote_access import start_remote_access_service
 from .tornado import open_webbrowser_on_server_started
-from .utils import make_applications, render_page, cdn_validation
+from .utils import make_applications, render_page, cdn_validation, deserialize_binary_event
 from ..session import CoroutineBasedSession, ThreadBasedSession, register_session_implement_for_target, Session
 from ..session.base import get_session_info_from_headers
 from ..utils import get_free_port, STATIC_PATH, iscoroutinefunction, isgeneratorfunction
@@ -41,7 +42,7 @@ def _webio_handler(applications, cdn, websocket_settings, check_origin_func=_is_
     """
     :param dict applications: dict of `name -> task function`
     :param bool/str cdn: Whether to load front-end static resources from CDN
-    :param callable check_origin_func: check_origin_func(origin, handler) -> bool
+    :param callable check_origin_func: check_origin_func(origin, host) -> bool
     :return: aiohttp Request Handler
     """
     ioloop = asyncio.get_event_loop()
@@ -81,6 +82,7 @@ def _webio_handler(applications, cdn, websocket_settings, check_origin_func=_is_
         session_info['user_ip'] = request.remote
         session_info['request'] = request
         session_info['backend'] = 'aiohttp'
+        session_info['protocol'] = 'websocket'
 
         app_name = request.query.getone('app', 'index')
         application = applications.get(app_name) or applications['index']
@@ -94,18 +96,23 @@ def _webio_handler(applications, cdn, websocket_settings, check_origin_func=_is_
                                          on_task_command=send_msg_to_client,
                                          on_session_close=close_from_session, loop=ioloop)
 
-        async for msg in ws:
-            if msg.type == web.WSMsgType.text:
-                data = msg.json()
+        # see: https://github.com/aio-libs/aiohttp/issues/1768
+        try:
+            async for msg in ws:
+                if msg.type == web.WSMsgType.text:
+                    data = msg.json()
+                elif msg.type == web.WSMsgType.binary:
+                    data = deserialize_binary_event(msg.data)
+                elif msg.type == web.WSMsgType.close:
+                    raise asyncio.CancelledError()
+
                 if data is not None:
                     session.send_client_event(data)
-            elif msg.type == web.WSMsgType.binary:
-                pass
-            elif msg.type == web.WSMsgType.close:
-                if not close_from_session_tag:
-                    # close session because client disconnected to server
-                    session.close(nonblock=True)
-                    logger.debug("WebSocket closed from client")
+        finally:
+            if not close_from_session_tag:
+                # close session because client disconnected to server
+                session.close(nonblock=True)
+                logger.debug("WebSocket closed from client")
 
         return ws
 
@@ -131,9 +138,9 @@ def webio_handler(applications, cdn=True, allowed_origins=None, check_origin=Non
     if check_origin is None:
         check_origin_func = partial(_check_origin, allowed_origins=allowed_origins or [])
     else:
-        check_origin_func = lambda origin, handler: _is_same_site(origin, handler) or check_origin(origin)
+        check_origin_func = lambda origin, host: _is_same_site(origin, host) or check_origin(origin)
 
-    return _webio_handler(applications=applications,cdn=cdn,
+    return _webio_handler(applications=applications, cdn=cdn,
                           check_origin_func=check_origin_func,
                           websocket_settings=websocket_settings)
 
@@ -157,7 +164,7 @@ def static_routes(prefix='/'):
 
 
 def start_server(applications, port=0, host='', debug=False,
-                 cdn=True, static_dir=None,
+                 cdn=True, static_dir=None, remote_access=False,
                  allowed_origins=None, check_origin=None,
                  auto_open_webbrowser=False,
                  websocket_settings=None,
@@ -197,4 +204,9 @@ def start_server(applications, port=0, host='', debug=False,
         logging.getLogger("asyncio").setLevel(logging.DEBUG)
 
     print('Listen on %s:%s' % (host, port))
+
+    if remote_access or remote_access == {}:
+        if remote_access is True: remote_access = {}
+        start_remote_access_service(**remote_access, local_port=port)
+
     web.run_app(app, host=host, port=port)
