@@ -113,7 +113,7 @@ r"""
        * ``origin`` (str): Indicate where the user from. Including protocol, host, and port parts. Such as ``'http://localhost:8080'`` .
          It may be empty, but it is guaranteed to have a value when the user's page address is not under the server host. (that is, the host, port part are inconsistent with ``server_host``).
        * ``user_ip`` (str): User's ip address.
-       * ``backend`` (str): The current PyWebIO backend server implementation. The possible values are ``'tornado'``, ``'flask'``, ``'django'`` , ``'aiohttp'``.
+       * ``backend`` (str): The current PyWebIO backend server implementation. The possible values are ``'tornado'``, ``'flask'``, ``'django'`` , ``'aiohttp'`` , ``'starlette'``.
        * ``protocol`` (str): The communication protocol between PyWebIO server and browser. The possible values are ``'websocket'``, ``'http'``
        * ``request`` (object): The request object when creating the current session. Depending on the backend server, the type of ``request`` can be:
 
@@ -122,6 +122,7 @@ r"""
             * When using Flask, ``request`` is instance of `flask.Request <https://flask.palletsprojects.com/en/1.1.x/api/#incoming-request-data>`_
             * When using Django, ``request`` is instance of `django.http.HttpRequest <https://docs.djangoproject.com/en/3.0/ref/request-response/#django.http.HttpRequest>`_
             * When using aiohttp, ``request`` is instance of `aiohttp.web.BaseRequest <https://docs.aiohttp.org/en/stable/web_reference.html#aiohttp.web.BaseRequest>`_
+            * When using FastAPI/Starlette, ``request`` is instance of `starlette.websockets.WebSocket <https://www.starlette.io/websockets/>`_
 
     The ``user_agent`` attribute of the session information object is parsed by the user-agents library. See https://github.com/selwin/python-user-agents#usage
 
@@ -141,7 +142,7 @@ r"""
         put_code(json.dumps({
             k: str(getattr(session_info, k))
             for k in ['user_agent', 'user_language', 'server_host',
-                      'origin', 'user_ip', 'backend', 'request']
+                      'origin', 'user_ip', 'backend', 'protocol', 'request']
         }, indent=4), 'json')
 
 .. autoclass:: pywebio.session.coroutinebased.TaskHandler
@@ -339,6 +340,8 @@ def eval_js(expression_, **args):
     """Execute JavaScript expression in the user's browser and get the value of the expression
 
     :param str expression_: JavaScript expression. The value of the expression need to be JSON-serializable.
+       If the value of the expression is a `promise <https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise>`_,
+       ``eval_js()`` will wait for the promise to resolve and return the value of it. When the promise is rejected, `None` is returned.
     :param args: Local variables passed to js code. Variables need to be JSON-serializable.
     :return: The value of the expression.
 
@@ -361,22 +364,21 @@ def eval_js(expression_, **args):
         })()''', b=100)
         put_text(function_res)  # ..demo-only
 
-    """
-    script = r"""
-    (function(WebIO){
-        let ____result____ = null;  // to avoid naming conflict
-        try{
-            ____result____ = eval(%r);
-        }catch{};
-        
-        WebIO.sendMessage({
-            event: "js_yield",
-            task_id: WebIOCurrentTaskID,  // local var in run_script command
-            data: ____result____ || null
-        });
-    })(WebIO);""" % expression_
+        ## ----
+        promise_res = eval_js('''new Promise(resolve => {
+            setTimeout(() => {
+                resolve('Returned inside callback.');
+            }, 2000);
+        });''')
+        put_text(promise_res)  # ..demo-only
 
-    run_js(script, **args)
+    .. versionchanged:: 1.3
+
+       The JS expression support return promise.
+    """
+
+    from ..io_ctrl import send_msg
+    send_msg('run_script', spec=dict(code=expression_, args=args, eval=True))
 
     res = yield next_client_event()
     assert res['event'] == 'js_yield', "Internal Error, please report this bug on " \
@@ -467,7 +469,7 @@ def data():
     global local
 
     import warnings
-    warnings.warn("`pywebio.session.data()` is deprecated in v1.1 and will remove in the future version, "
+    warnings.warn("`pywebio.session.data()` is deprecated since v1.1 and will remove in the future version, "
                   "use `pywebio.session.local` instead", DeprecationWarning, stacklevel=2)
     return local
 
@@ -481,13 +483,18 @@ def set_env(**env_info):
     * ``output_animation`` (bool): Whether to enable output animation, enabled by default
     * ``auto_scroll_bottom`` (bool): Whether to automatically scroll the page to the bottom after output content, it is closed by default.  Note that after enabled, only outputting to ROOT scope can trigger automatic scrolling.
     * ``http_pull_interval`` (int): The period of HTTP polling messages (in milliseconds, default 1000ms), only available in sessions based on HTTP connection.
+    * ``input_panel_fixed`` (bool): Whether to make input panel fixed at bottom, enabled by default
+    * ``input_panel_min_height`` (int): The minimum height of input panel (in pixel, default 300px), it should be larger than 75px. Available only when ``input_panel_fixed=True``
+    * ``input_panel_init_height`` (int): The initial height of input panel (in pixel, default 300px), it should be larger than 175px. Available only when ``input_panel_fixed=True``
+    * ``input_auto_focus`` (bool): Whether to focus on input automatically after showing input panel, default is ``True``
 
     Example::
 
         set_env(title='Awesome PyWebIO!!', output_animation=False)
     """
     from ..io_ctrl import send_msg
-    assert all(k in ('title', 'output_animation', 'auto_scroll_bottom', 'http_pull_interval')
+    assert all(k in ('title', 'output_animation', 'auto_scroll_bottom', 'http_pull_interval',
+                     'input_panel_min_height', 'input_panel_init_height', 'input_panel_fixed', 'input_auto_focus')
                for k in env_info.keys())
     send_msg('set_env', spec=env_info)
 
@@ -504,7 +511,7 @@ def go_app(name, new_window=True):
 
 
 # session info data object
-info = ReadOnlyObjectDict(lambda: get_current_session().info)  # type: _SessionInfoType
+info = ReadOnlyObjectDict(lambda: get_current_session().internal_save['info'])  # type: _SessionInfoType
 
 
 class _SessionInfoType:
@@ -527,6 +534,6 @@ def get_info():
     global info
 
     import warnings
-    warnings.warn("`pywebio.session.get_info()` is deprecated in v1.2 and will remove in the future version, "
+    warnings.warn("`pywebio.session.get_info()` is deprecated since v1.2 and will remove in the future version, "
                   "please use `pywebio.session.info` instead", DeprecationWarning, stacklevel=2)
     return info
